#ifndef SIMULATION_HPP
#define SIMULATION_HPP

#include <vector>
#include <memory>

#include "BarnesHutTree.hpp"
#include "Body.hpp"

namespace nbody {
	//simulation superclass
	class Simulation {
	protected:
		int parallelSize{ -1 };
		int parallelRank{ -1 };
		bool correctState{ false };
		std::vector<Body> bodies;
		std::unique_ptr<BarnesHutTree> tree;
	public:
		virtual void clearBodies();
		virtual std::vector<Body> getBodies() const;
		virtual std::size_t getNumberOfProcesses() const = 0;
		virtual std::size_t getProcessId() const = 0;
		virtual bool readInputData(const std::string& filename);
		virtual void runStep() = 0;
	};
} // namespace nbody

#endif
